% RecM_demo
% Demo script for the tomographic reconstruction problem in the matrix form
%
% This script illustrates the use of the reconstruction algorithm
% implemented in the functions: RecM_Algorithm, Linear_Opr, L_matrix,
% Perm_Vec.
% The user can easily modify the script for
% other tests problems, patch sizes, regularization parameters etc...
% These code handels one dictionary for a specific lambda.
% The codes need to be modified if one wants to solve the reconstruction
% problem for various values of lambda at the same time
% This codes uses the TFOCS optimization solver version 1.3.1
% (http://cvxr.com/tfocs/)
% make sure TFOCS is properly installed on your machine

% Inputs:
% TrImage: The test image, options are: 'Peppers', 'Matches', 
% 'Binary', 'Zirconium', 'Steel', 'D53'
% patch_size: The patch sizes of the blocks in the image, note that 
% the image size is a multiple of the patch size.
% D: The given(learned) dictionary, note that the dictionary should be
% obtained from a training image similar to the test image
% N_p: Number of tomographic projections 
% Ra: Range of the angles for the tomographic projections, e.g., full-range
% [0, 180], limited-angle [0 120]
% rnl: Guassian additive noise level in the tomographic data
% geo: Choose a parallel-beam CT geometry or a fan-beam CT geometry,
% options= 'fanbeam' and 'parallelbeam'. % default value = 'parallelbeam'
% Outdisplay: The display option on the screen, when set to 0 no display 
% on the screen, when set to 1 printes progress of the algorithm
% default = 0,


% Output: 
% TrI: The exact image considered in the tomographic problem. The size of
% the TrI is predefined.
% X_sol: The tomographic reconstruction solution recovered from Alpha
% Alpha: The sparse representation/solution in the given dictionary
% obtained by solving our tomographic reconstruction problem
% Error_Sol: Relative reconstruction error

% Copyright (C) Sara Soltani and Technical University of Denmark, 
% August 2015, ssol@dtu.dk, sarahsoltani@gmail.com


clear all;
close all;
clc;

% Set the necessary paths
addpath /home/ssol/TFOCS-1.3.1
addpath /home/ssol/AIRtools/AIRtools

addpath /home/ssol/DLCT-ToolBox/TestImages
addpath /home/ssol/DLCT-ToolBox/TestResults

% Set the test problem
% The test image, options are: 'Peppers', 'Matches', 
% 'Binary', 'Zirconium', 'Steel', 'D53'
TrImage = 'Peppers';

% Load the desired dictionary, note that the dictionary should be obtained
% from a similar training image
load('Dic.mat','D');

% Note that the patch size of the dictionary and the blocks in
% the image should be equal 
patch_size = [5,5]; 

% Set the tompgraphic problem
Ra = 180;
N_p = 25;
rnl = 0.01; 
geo = 'fanbeam';

% The display option on the screen
% When set to 0 no display on the screen
% When set to 1 printes progress of the algorithm
% default = 0,
Outdisplay = 1;

% Solve the tomographic reconstruction problem

[TrI,X_sol,Alph,Error_Sol] = ...
RecM_Algorithm(TrImage,D,patch_size,N_p,Ra,rnl,geo,Outdisplay);

BestError=min(min(Error_Sol));
[I,J] = find(Error_Sol==BestError);
figure, imagesc(TrI,[0 1]); axis image, colormap gray
figure, imagesc(reshape(X_sol(:,I,J),size(TrI,1),size(TrI,2)),[0 1]);
axis image, colormap gray